/*
* Smart IR I2C interface
*/

#include "smartir2iic.h"
#include "Arduino.h"
#include "Wire.h"

#define _EC_OK 200
#define _PREFIX 0xAA

smartir2iic::smartir2iic(uint8_t addr) {
  if(addr>8 && addr<120) _addr = addr;
}

void smartir2iic::begin(void) {
  Wire.begin();
  reboot();
  delay(500);
  _p0r5x(0x31);
  _addr  = _arr[3];
  _led   = _arr[6];
  _light = _arr[7];
}

uint8_t smartir2iic::cmd(void) {
  _p0r1(0x01);
  return(_arr[3]);
}

uint8_t smartir2iic::rc(void) {
  _p0r2(0x02);
  return(_arr[3]);
}

uint32_t smartir2iic::code(void) {
  uint32_t r;
  _p0r4(0x03);
  r = _arr[3]; r<<=8; r+=_arr[4]; r<<=8; r+=_arr[5]; r<<=8; r+=_arr[6];
  return(r);
}

uint8_t smartir2iic::protocol(void) {
  _p0r1(0x04);
  return(_arr[3]);
}

bool smartir2iic::create(uint8_t rc, uint8_t cmd) {
  _p2r1(0x11,rc,cmd);
  return(_arr[3]==_EC_OK?true:false);
}

bool smartir2iic::edit(uint8_t rc, uint8_t cmd) {
  _p2r1(0x12,rc,cmd);
  return(_arr[3]==_EC_OK?true:false);
}

bool smartir2iic::setModeLed(uint8_t stat) {
  if(stat<3) {
    _p1r1(0x13,stat);
    return(_arr[3]==_EC_OK?true:false);
  } return(false);
}

bool smartir2iic::setLightLed(uint8_t light) {
  _p1r1(0x14,light);
  return(_arr[3]==_EC_OK?true:false);
}

uint8_t smartir2iic::tCode(uint8_t *data) {
  _p0rN(0x21, data);
  return(_arr[2]);
}

bool smartir2iic::decode1(uint8_t m0, uint8_t s0, uint8_t m1, uint8_t s1, uint8_t offset, uint8_t ln) {
  _arr[0] = _PREFIX;
  _arr[1] = 0x22; _arr[2] = 7;
  _arr[3] = 1;
  _arr[4] = m0; _arr[5] = s0; _arr[6] = m1; _arr[7] = s1;
  _arr[8] = offset; _arr[9] = ln;
  _arr[10] = _chk_s(_arr,10);
  Wire.beginTransmission(_addr);
  Wire.write(_arr,11);
  Wire.endTransmission();
  Wire.requestFrom(_addr,5,true);
  if(Wire.available()>0) Wire.readBytes(_arr,5);
  return(_arr[3]==_EC_OK?true:false);
}

bool smartir2iic::decode2(bool phase, uint8_t offset, uint8_t ln) {
  _arr[0] = _PREFIX;
  _arr[1] = 0x22; _arr[2] = 4;
  _arr[3] = 2;
  _arr[4] = phase?1:0;
  _arr[5] = offset; _arr[6] = ln;
  _arr[7] = _chk_s(_arr,7);
  Wire.beginTransmission(_addr);
  Wire.write(_arr,8);
  Wire.endTransmission();
  Wire.requestFrom(_addr,5,true);
  if(Wire.available()>0) Wire.readBytes(_arr,5);
  return(_arr[3]==_EC_OK?true:false);
}

uint8_t smartir2iic::addr(void) {
  return(_addr);
}

uint32_t smartir2iic::speed(void) {
  return(_speed);
}

uint8_t smartir2iic::modeLed(void) {
  return(_led);
}

uint8_t smartir2iic::lightLed(void) {
  return(_light);
}

bool smartir2iic::saveConf(uint8_t addr, uint32_t speed, uint8_t led, uint8_t light) {
  _arr[5] = 0xff;
  bool r = false;
  if(led<3) {
    if(speed==19200) _arr[5] = 0;
    if(speed==38400) _arr[5] = 1;
    if(speed==57600) _arr[5] = 2;
    if(speed==115200) _arr[5] = 3;
    if(speed==230400) _arr[5] = 4;
    if(speed==460800) _arr[5] = 5;
    if(speed==921600) _arr[5] = 6;
    if(_arr[5]!=0xff) {
      _arr[0] = _PREFIX;
      _arr[1] = 0x3e; _arr[2] = 5;
      _arr[3] = addr; _arr[4] = 0; _arr[6] = led; _arr[7] = light;
      _arr[8] = _chk_s(_arr,8);
      
      Wire.beginTransmission(_addr);
      Wire.write(_arr,9);
      Wire.endTransmission();
      Wire.requestFrom(_addr,5,true);
      if(Wire.available()>0) Wire.readBytes(_arr,5);
      r=(_arr[3]==_EC_OK) ?true:false;
    }
  }
  return(r);
}

bool smartir2iic::clearConf(void) {
  _p0r1(0x3f);
  return(_arr[3]==_EC_OK?true:false);
}
  
bool smartir2iic::saveCode(void) {
  _p0r1(0x4e);
  return(_arr[3]==_EC_OK?true:false);
}
  
bool smartir2iic::clearCode(void) {
  _p0r1(0x4f);
  return(_arr[3]==_EC_OK?true:false);
}

uint8_t smartir2iic::name(uint8_t *data) {
  _p0r8(0x52, data);
  data[8] = 0;
  return(_arr[3]);
}

uint8_t smartir2iic::version(uint8_t *data) {
  _p0r8(0x53, data);
  data[8] = 0;
  return(_arr[3]);
}

void smartir2iic::reboot(void) {
  _p0r0(0xfe);
}

/*** Subroutines ***/
void smartir2iic::_p0r0(uint8_t cmd) {
  _arr[0] = _PREFIX;
  _arr[1] = cmd; _arr[2] = 0;
  _arr[3] = _chk_s(_arr,3);
  Wire.beginTransmission(_addr);
  Wire.write(_arr,4);
  Wire.endTransmission();
}

void smartir2iic::_p0r1(uint8_t cmd) {
  while(1) {
    _p0r0(cmd);
    Wire.requestFrom(_addr,5,true);
    if(Wire.available()>0) Wire.readBytes(_arr,5);
    if(_arr[4]==_chk_s(_arr,4)) break;
  }
}
void smartir2iic::_p0r2(uint8_t cmd) {
  while(1) {
    _p0r0(cmd);
    Wire.requestFrom(_addr,6,true);
    if(Wire.available()>0) Wire.readBytes(_arr,6);
    if(_arr[5]==_chk_s(_arr,5)) break;
  }
}

void smartir2iic::_p0r4(uint8_t cmd) {
  while(1) {
    _p0r0(cmd);
    Wire.requestFrom(_addr,8,true);
    if(Wire.available()>0) Wire.readBytes(_arr,8);
    if(_arr[7]==_chk_s(_arr,7)) break;
  }
}

void smartir2iic::_p0r5x(uint8_t cmd) {
  while(1) {
    _p0r0(cmd);
    Wire.requestFrom(_addr,9,true);
    if(Wire.available()>0) Wire.readBytes(_arr,9);
    if(_arr[8]==_chk_s(_arr,8)) break;
  }
}

void smartir2iic::_p0r8(uint8_t cmd, uint8_t *data) {
  while(1) {
    _p0r0(cmd);
    Wire.requestFrom(_addr,12,true);
    if(Wire.available()>0) Wire.readBytes(_arr,12);
    for(int i=0;i<8;i++) data[i] = _arr[3+i];
    if(_arr[11]==_chk_s(_arr,11)) break;
  }
}

void smartir2iic::_p1r1(uint8_t cmd, uint8_t d1) {
  while(1) {
    _arr[0] = _PREFIX;
    _arr[1] = cmd; _arr[2] = 1; _arr[3] = d1;
    _arr[4] = _chk_s(_arr,4);
    Wire.beginTransmission(_addr);
    Wire.write(_arr,5);
    Wire.endTransmission();
    Wire.requestFrom(_addr,5,true);
    if(Wire.available()>0) Wire.readBytes(_arr,5);
    if(_arr[4]==_chk_s(_arr,4)) break;
  }
}

void smartir2iic::_p2r1(uint8_t cmd, uint8_t d1, uint8_t d2) {
  while(1) {
    _arr[0] = _PREFIX;
    _arr[1] = cmd; _arr[2] = 2;
    _arr[3] = d1; _arr[4] = d2;
    _arr[5] = _chk_s(_arr,5);
    Wire.beginTransmission(_addr);
    Wire.write(_arr,6);
    Wire.endTransmission();
    Wire.requestFrom(_addr,5,true);
    if(Wire.available()>0) Wire.readBytes(_arr,5);
    if(_arr[4]==_chk_s(_arr,4)) break;
  }
}

void smartir2iic::_p0rN(uint8_t cmd, uint8_t *data) {
  while(1) {
    _p0r0(cmd);
    Wire.requestFrom(data,3,false);
    if(Wire.available()>0) Wire.readBytes(data,3);
    _arr[2] = data[2];
    Wire.requestFrom(data+3,data[2]+1,true);
    if(Wire.available()>0) Wire.readBytes(data+3,data[2]+1);
    if(data[_arr[2]+3]==_chk_s(data,_arr[2]+3)) break;
  }
  for(int i=0;i<_arr[2];i++) data[i] = data[i+3];
}

void smartir2iic::_p4r1(uint8_t cmd, uint8_t *data) {
  while(1) {
    _arr[0] = _PREFIX;
    _arr[1] = cmd; _arr[2] = 4;
    for(int i;i<4;i++)
      _arr[3+i] = data[i];
    _arr[7] = _chk_s(_arr,7);
    Wire.beginTransmission(_addr);
    Wire.write(_arr,8);
    Wire.endTransmission();
    Wire.requestFrom(_addr,5,true);
    if(Wire.available()>0) Wire.readBytes(_arr,5);
    if(_arr[4]==_chk_s(_arr,4)) break;
  }
}

void smartir2iic::_p7r1(uint8_t cmd, uint8_t *data) {
  while(1) {
    _arr[0] = _PREFIX;
    _arr[1] = cmd; _arr[2] = 4;
    for(int i;i<7;i++)
      _arr[3+i] = data[i];
    _arr[10] = _chk_s(_arr,10);
    Wire.beginTransmission(_addr);
    Wire.write(_arr,11);
    Wire.endTransmission();
    Wire.requestFrom(_addr,5,true);
    if(Wire.available()>0) Wire.readBytes(_arr,5);
    if(_arr[4]==_chk_s(_arr,4)) break;
  }
}

uint8_t smartir2iic::_chk_s(uint8_t *a, uint8_t n) {
  int x = 0;
  for (int i=0;i<n;i++) x+=a[i];
  return(lowByte(x));
}



